@@ -16,6 +16,7 @@ import tempfile
 import subprocess
 import time
 import importlib.util
+import tracemalloc
 
 import dbus
 import dbus.mainloop.glib
@@ -24,6 +25,7 @@ from gi.repository import GLib
 
 import dbusmock
 
+tracemalloc.start(25)
 dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
 
 # "a <heart> b" in py2/3 compatible unicode
@@ -336,6 +338,43 @@ assert args[2] == 5
         self.assertEqual(self.dbus_props.Get('org.freedesktop.Test.Other', 'color'),
                          'yellow')
 
+        changed_props = []
+        ml = GLib.MainLoop()
+
+        def catch(*args, **kwargs):
+            if kwargs['interface'] != 'org.freedesktop.DBus.Properties':
+                return
+
+            self.assertEqual(kwargs['interface'], 'org.freedesktop.DBus.Properties')
+            self.assertEqual(kwargs['member'], 'PropertiesChanged')
+
+            [iface, changed, _invalidated] = args
+            self.assertEqual(iface, 'org.freedesktop.Test.Main')
+
+            changed_props.append(changed)
+            ml.quit()
+
+        match = self.dbus_con.add_signal_receiver(catch,
+                                                  interface_keyword='interface',
+                                                  path_keyword='path',
+                                                  member_keyword='member')
+
+        # change property using mock helper
+        self.dbus_mock.UpdateProperties('org.freedesktop.Test.Main', {
+            'version': 5,
+            'connected': False,
+        })
+
+        GLib.timeout_add(3000, ml.quit)
+        ml.run()
+
+        match.remove()
+
+        self.assertEqual(self.dbus_props.GetAll('org.freedesktop.Test.Main'),
+                         {'version': 5, 'connected': False})
+        self.assertEqual(changed_props,
+                         [{'version': 5, 'connected': False}])
+
     def test_introspection_methods(self):
         '''dynamically added methods appear in introspection'''
 
@@ -667,6 +706,51 @@ def Answer(self):
     def test_local_nonexisting(self):
         self.assertRaises(ImportError, self.spawn_server_template, '/non/existing.py')
 
+    def test_explicit_bus_(self):
+        '''Explicitly set the bus for a template that does not specify SYSTEM_BUS'''
+
+        with tempfile.NamedTemporaryFile(prefix='answer_', suffix='.py') as my_template:
+            my_template.write(b'''import dbus
+BUS_NAME = 'universe.Ultimate'
+MAIN_OBJ = '/'
+MAIN_IFACE = 'universe.Ultimate'
+
+def load(mock, parameters):
+    mock.AddMethods(MAIN_IFACE, [('Answer', '', 'i', 'ret = 42')])
+''')
+            my_template.flush()
+            (p_mock, dbus_ultimate) = self.spawn_server_template(
+                my_template.name, stdout=subprocess.PIPE, system_bus=False)
+            self.addCleanup(p_mock.wait)
+            self.addCleanup(p_mock.terminate)
+            self.addCleanup(p_mock.stdout.close)
+
+        self.wait_for_bus_object('universe.Ultimate', '/')
+        self.assertEqual(dbus_ultimate.Answer(), 42)
+
+    def test_override_bus_(self):
+        '''Override the bus for a template'''
+
+        with tempfile.NamedTemporaryFile(prefix='answer_', suffix='.py') as my_template:
+            my_template.write(b'''import dbus
+BUS_NAME = 'universe.Ultimate'
+MAIN_OBJ = '/'
+MAIN_IFACE = 'universe.Ultimate'
+SYSTEM_BUS = True
+
+def load(mock, parameters):
+    mock.AddMethods(MAIN_IFACE, [('Answer', '', 'i', 'ret = 42')])
+''')
+            my_template.flush()
+            (p_mock, dbus_ultimate) = self.spawn_server_template(
+                my_template.name, stdout=subprocess.PIPE, system_bus=False)
+            self.addCleanup(p_mock.wait)
+            self.addCleanup(p_mock.terminate)
+            self.addCleanup(p_mock.stdout.close)
+
+        self.wait_for_bus_object('universe.Ultimate', '/')
+        self.assertEqual(dbus_ultimate.Answer(), 42)
+
     def test_object_manager(self):
         '''Template with ObjectManager API'''
 
